@@ -20,7 +20,7 @@ use crate::errors::file::FileError;
 use crate::{Error, Result};
 
 #[cfg(doc)]
-use crate::errors::{encryption::EncryptionError, file::FileError, io::IoError};
+use crate::errors::{encryption::EncryptionError, io::IoError};
 
 /// Allowed file size range in bytes
 const ALLOWED_FILE_SIZE_RANGE: RangeInclusive<u64> = 20..=50_000;
@@ -618,6 +618,22 @@ pub struct File {
     bytes: Vec<u8>,
     /// A custom file name
     name: Option<String>,
+    /// File should be discarded when dropped
+    is_temp: bool,
+}
+
+impl Drop for File {
+    fn drop(&mut self) {
+        if self.is_temp {
+            if let Err(err) = std::fs::remove_file(&self.path) {
+                eprintln!(
+                    "couldn't delete temp file {:?}\n{}",
+                    self.path,
+                    err.to_string()
+                );
+            }
+        }
+    }
 }
 
 #[must_use]
@@ -629,8 +645,8 @@ pub fn name_from_path(path: &Path) -> String {
         .to_string()
 }
 
-async fn spawn_and_write_rng_file(bytes: &[u8]) -> Result<(tokio::fs::File, PathBuf)> {
-    let path = rng_temp_file();
+async fn rng_file_with_name(bytes: &[u8], name: &str) -> Result<(tokio::fs::File, PathBuf)> {
+    let path = rng_temp_file(name);
     let mut handler = tokio::fs::File::create(&path).await?;
     handler.write_all(bytes).await?;
     Ok((handler, path))
@@ -652,7 +668,7 @@ impl File {
             .map_err(|err| {
                 // Intercept OS error 21 (EISDIR), meaning we can't write to a directory
                 #[cfg(target_family = "unix")]
-                if let Some(21) = err.raw_os_error() {
+                if err.raw_os_error() == Some(21) {
                     Error::File(FileError::NotAFile(name_from_path(path)))
                 } else {
                     Error::from(err)
@@ -672,6 +688,7 @@ impl File {
             path: path.to_path_buf(),
             bytes,
             name: Some(name_from_path(path)),
+            is_temp: false,
         })
     }
 
@@ -682,32 +699,34 @@ impl File {
     ///
     /// Fails with [`IoError`] if the file can't be created for some reason. Also if it can't be
     /// written to.
-    pub async fn from_bytes_encoded(bytes: &[u8]) -> Result<Self> {
+    pub async fn from_bytes_encoded(bytes: &[u8], name: &str) -> Result<Self> {
         let decoded_bytes = base64::decode(bytes)?;
-        let (handler, path) = spawn_and_write_rng_file(&decoded_bytes).await?;
+        let (handler, path) = rng_file_with_name(&decoded_bytes, name).await?;
 
         Ok(Self {
             handler,
             name: Some(name_from_path(&path)),
             path,
             bytes: decoded_bytes,
+            is_temp: true,
         })
     }
 
-    /// Same as [`from_bytes_encoded`] but doesn't expect encoded bytes
+    /// Same as [`Self::from_bytes_encoded`] but doesn't expect encoded bytes
     ///
     /// # Errors
     ///
     /// Fails with [`IoError`] if the file can't be created for some reason. Also if it can't be
     /// written to.
     pub async fn from_bytes(decoded_bytes: &[u8]) -> Result<Self> {
-        let (handler, path) = spawn_and_write_rng_file(decoded_bytes).await?;
+        let (handler, path) = rng_file_with_name(decoded_bytes, "").await?;
 
         Ok(Self {
             handler,
             name: Some(name_from_path(&path)),
             path,
             bytes: decoded_bytes.to_vec(),
+            is_temp: true,
         })
     }
 
@@ -720,15 +739,13 @@ impl File {
         Ok(tokio::fs::write(file_path, &self.bytes).await?)
     }
 
-    /// Set the file name, useful when creating a [`File`] using [`from_bytes`].
-    /// if the [`File`] was created using [`from_path`] it will use the provided file name.
+    /// Set the file name, useful when creating a [`File`] using [`Self::from_bytes`].
+    /// if the [`File`] was created using [`Self::from_path`] it will use the provided file name.
     #[allow(clippy::missing_const_for_fn)]
     #[must_use]
-    pub fn with_name(self, name: String) -> Self {
-        Self {
-            name: Some(name),
-            ..self
-        }
+    pub fn with_name(mut self, name: &str) -> Self {
+        self.name = Some(name.to_owned());
+        self
     }
 
     /// Returns the file name
@@ -763,15 +780,15 @@ impl File {
     /// # Errors
     ///
     /// Fails with [`EncryptionError`] if something goes wrong during the encryption process. This
-    /// includes unexpected sizes of the nounce, hashed key.
-    /// Will also error out if the provided key and nounce is incorrect.
+    /// includes unexpected sizes of the nonce, hashed key.
+    /// Will also error out if the provided key and nonce is incorrect.
     pub async fn into_encrypted(self, secret: &str) -> Result<EncryptedFile> {
-        let (encrypted_bytes, nounce) = encrypt_aes256_u12nonce(secret.as_bytes(), self.data())?;
+        let (encrypted_bytes, nonce) = encrypt_aes256_u12nonce(secret.as_bytes(), self.data())?;
         let name = self.name.clone();
 
         Ok(EncryptedFile {
             encrypted_bytes,
-            nounce,
+            nonce,
             prev: Some(Box::new(self)),
             name,
         })
@@ -791,7 +808,7 @@ impl File {
 }
 
 /// Represents a encrypted gistit file data.
-/// This data structure is expected to hold encrypted but not encoded bytes in `encrypted_bytes`, the `nounce`
+/// This data structure is expected to hold encrypted but not encoded bytes in `encrypted_bytes`, the `nonce`
 /// which is a 12 bytes randomly generated byte array, and a pointer to the previous unencrypted [`File`]
 /// handler.
 #[derive(Debug)]
@@ -799,25 +816,25 @@ pub struct EncryptedFile {
     /// The encrypted bytes
     encrypted_bytes: Vec<u8>,
     /// The random sequence used to encrypt
-    nounce: Vec<u8>,
+    nonce: Vec<u8>,
     /// Pointer to maybe the previous unencrypted
     prev: Option<Box<File>>,
     /// Overwrite the random file name during decryption
     name: Option<String>,
 }
 
-/// Extract and verify the encrypted file header which contains the `nounce` and a expected 8 bytes
+/// Extract and verify the encrypted file header which contains the `nonce` and a expected 8 bytes
 /// long padding defined in [`FILE_HEADER_ENCRYPTION_PADDING`].
 ///
 /// # Errors
 ///
-/// Fails with [`FileError`] if the padding is invalid or the `nounce` is incorrectly sized.
+/// Fails with [`FileError`] if the padding is invalid or the `nonce` is incorrectly sized.
 fn parse_encryption_header(bytes: &[u8]) -> Result<(Vec<u8>, Vec<u8>)> {
     let (header, rest) = bytes.split_at(20);
-    let (nounce, padding) = header.split_at(12);
+    let (nonce, padding) = header.split_at(12);
 
     if padding == FILE_HEADER_ENCRYPTION_PADDING.as_bytes() {
-        Ok((nounce.to_vec(), rest.to_vec()))
+        Ok((nonce.to_vec(), rest.to_vec()))
     } else {
         Err(Error::File(FileError::InvalidEncryptionPadding))
     }
@@ -836,17 +853,17 @@ impl EncryptedFile {
     ///
     /// Fails with [`IoError`] if can't create or write to the file handler.
     /// Fails with [`FileError`] if the encryption header is invalid.
-    pub async fn from_bytes_encoded(encoded_bytes: &[u8]) -> Result<Self> {
+    pub async fn from_bytes_encoded(encoded_bytes: &[u8], name: &str) -> Result<Self> {
         let decoded_bytes = base64::decode(encoded_bytes)?;
-        let path = rng_temp_file();
+        let path = rng_temp_file(name);
 
-        let (nounce, encrypted_bytes) = parse_encryption_header(&decoded_bytes)?;
+        let (nonce, encrypted_bytes) = parse_encryption_header(&decoded_bytes)?;
         let mut handler = tokio::fs::File::create(&path).await?;
         handler.write_all(&encrypted_bytes).await?;
 
         Ok(Self {
             encrypted_bytes,
-            nounce,
+            nonce,
             prev: None,
             name: Some(name_from_path(&path)),
         })
@@ -856,11 +873,9 @@ impl EncryptedFile {
     /// string.
     #[allow(clippy::missing_const_for_fn)]
     #[must_use]
-    pub fn with_name(self, name: String) -> Self {
-        Self {
-            name: Some(name),
-            ..self
-        }
+    pub fn with_name(mut self, name: &str) -> Self {
+        self.name = Some(name.to_owned());
+        self
     }
 
     /// Converts [`Self`] into [`File`] handler by applying the decryption process with the
@@ -870,22 +885,24 @@ impl EncryptedFile {
     ///
     /// Fails with [`EncryptionError`] if `nonce` or `secret` is incorrect
     pub async fn into_decrypted(self, secret: &str) -> Result<File> {
-        let nounce: [u8; 12] = self
-            .nounce
+        let nonce: [u8; 12] = self
+            .nonce
             .clone()
             .try_into()
-            .expect("Shrink nounce to 12 bytes");
+            .expect("Shrink nonce to 12 bytes");
 
-        let decrypted_bytes = decrypt_aes256_u12nonce(secret.as_bytes(), self.data(), &nounce)?;
-        let file = File::from_bytes(&decrypted_bytes)
-            .await?
-            .with_name(self.name.expect("Opened file to have at least temp name"));
+        let decrypted_bytes = decrypt_aes256_u12nonce(secret.as_bytes(), self.data(), &nonce)?;
+        let file = File::from_bytes(&decrypted_bytes).await?.with_name(
+            self.name
+                .as_ref()
+                .expect("Opened file to have at least temp name"),
+        );
         Ok(file)
     }
 }
 
 /// Returns a new randomnly generated file path in your system `temp` directory
-fn rng_temp_file() -> PathBuf {
+fn rng_temp_file(suffix: &str) -> PathBuf {
     let rng_string: String = rand::thread_rng()
         .sample_iter(&Alphanumeric)
         .take(8)
@@ -894,6 +911,7 @@ fn rng_temp_file() -> PathBuf {
 
     let mut rng_file_name = "__gistit_tmp_".to_owned();
     rng_file_name.push_str(&rng_string);
+    rng_file_name.push_str(suffix);
 
     temp_dir().join(&rng_file_name)
 }
@@ -922,7 +940,7 @@ impl FileReady for EncryptedFile {
     }
 
     fn to_encoded_data(&self) -> EncodedFileData {
-        let mut headful_data = self.nounce.clone();
+        let mut headful_data = self.nonce.clone();
         headful_data.extend(FILE_HEADER_ENCRYPTION_PADDING.as_bytes());
         headful_data.extend_from_slice(&self.encrypted_bytes);
 
@@ -993,3 +1011,282 @@ impl Check for File {
         }
     }
 }
+
+#[cfg(test)]
+mod tests {
+    use super::*;
+    use crate::errors::encryption::EncryptionError;
+    use assert_fs::prelude::*;
+    use predicates::prelude::*;
+    use tokio::task::spawn_blocking;
+
+    #[test]
+    fn file_name_from_path_edge_cases() {
+        let n1 = name_from_path(Path::new("foo.txt"));
+        let n2 = name_from_path(Path::new("~/foo.txt"));
+        let n3 = name_from_path(Path::new("/foo.txt"));
+        let n4 = name_from_path(Path::new("/😁.txt"));
+        let n5 = name_from_path(Path::new("/lol/what/foo/😁.txt"));
+        let n6 = name_from_path(Path::new("😁"));
+
+        assert_eq!(n1, "foo.txt");
+        assert_eq!(n2, "foo.txt");
+        assert_eq!(n3, "foo.txt");
+        assert_eq!(n4, "😁.txt");
+        assert_eq!(n5, "😁.txt");
+        assert_eq!(n6, "😁");
+    }
+
+    #[tokio::test]
+    async fn file_spawn_random_and_write() {
+        let data: String = rand::thread_rng()
+            .sample_iter(&Alphanumeric)
+            .take(512)
+            .map(char::from)
+            .collect();
+        let (file, path) = rng_file_with_name(data.as_bytes(), "foo.txt")
+            .await
+            .unwrap();
+        let read_bytes = tokio::fs::read(path).await.unwrap();
+        assert_eq!(data, std::str::from_utf8(&read_bytes).unwrap());
+    }
+
+    #[tokio::test]
+    async fn file_structure_new_from_path() {
+        let data: String = rand::thread_rng()
+            .sample_iter(&Alphanumeric)
+            .take(512)
+            .map(char::from)
+            .collect();
+        let tmp = assert_fs::TempDir::new().unwrap();
+        let input_file = tmp.child("foo.txt");
+
+        input_file.touch().unwrap();
+        input_file.write_binary(data.as_bytes()).unwrap();
+
+        let file = File::from_path(&input_file).await.unwrap();
+        assert_eq!(file.size().await, 512);
+        assert_eq!(file.bytes, data.as_bytes());
+        assert_eq!(file.name(), "foo.txt".to_owned());
+    }
+
+    #[tokio::test]
+    async fn file_structure_new_from_path_fails_if_is_dir() {
+        let tmp = assert_fs::TempDir::new().unwrap();
+        let read_err = File::from_path(&tmp).await.unwrap_err();
+        assert!(matches!(read_err, Error::File(FileError::NotAFile(_))));
+    }
+
+    #[tokio::test]
+    async fn file_structure_new_from_bytes_encoded() {
+        let data: String = rand::thread_rng()
+            .sample_iter(&Alphanumeric)
+            .take(512)
+            .map(char::from)
+            .collect();
+        let encoded_data = base64::encode(&data);
+        let file = File::from_bytes_encoded(encoded_data.as_bytes(), "nameless")
+            .await
+            .unwrap();
+        assert_eq!(file.bytes, data.as_bytes());
+
+        // Fails when encoding is corrupted
+        let mut corrupted_data = "¨¨¨¨".to_owned();
+        corrupted_data.extend(encoded_data.chars());
+        let decode_err = File::from_bytes_encoded(corrupted_data.as_bytes(), "nameless")
+            .await
+            .unwrap_err();
+        assert!(matches!(
+            decode_err,
+            Error::Encryption(EncryptionError::Encoding(_))
+        ));
+    }
+
+    #[tokio::test]
+    async fn file_structure_new_from_bytes() {
+        let data: String = rand::thread_rng()
+            .sample_iter(&Alphanumeric)
+            .take(512)
+            .map(char::from)
+            .collect();
+        let file = File::from_bytes(data.as_bytes()).await.unwrap();
+        assert_eq!(file.bytes, data.as_bytes());
+    }
+
+    #[tokio::test]
+    async fn file_structure_save_as() {
+        let data: String = rand::thread_rng()
+            .sample_iter(&Alphanumeric)
+            .take(512)
+            .map(char::from)
+            .collect();
+        let tmp = assert_fs::TempDir::new().unwrap();
+        let tmp_file = tmp.child("foo.txt");
+
+        tmp_file.touch().unwrap();
+        tmp_file.write_binary(data.as_bytes()).unwrap();
+
+        let file = File::from_path(&tmp_file).await.unwrap();
+        file.save_as(&tmp.join("bar.txt")).await.unwrap();
+        tmp.assert(predicates::path::exists());
+
+        let other = tmp.child("bar.txt");
+        let other = tokio::fs::read(other).await.unwrap();
+        assert_eq!(data.as_bytes(), other);
+    }
+
+    #[tokio::test]
+    async fn file_structure_extension_to_lang_mapping() {
+        let tmp = assert_fs::TempDir::new().unwrap();
+
+        let rust = tmp.child("foo.rs");
+        let js = tmp.child("bar.js");
+        let cpp = tmp.child("lol.cpp");
+        let brainfuck = tmp.child("rly.bf");
+        rust.touch().unwrap();
+        js.touch().unwrap();
+        cpp.touch().unwrap();
+        brainfuck.touch().unwrap();
+
+        assert_eq!(File::from_path(&rust).await.unwrap().lang(), "rust");
+        assert_eq!(File::from_path(&js).await.unwrap().lang(), "javascript");
+        assert_eq!(File::from_path(&cpp).await.unwrap().lang(), "cpp");
+        assert_eq!(
+            File::from_path(&brainfuck).await.unwrap().lang(),
+            "brainfuck"
+        );
+    }
+
+    #[tokio::test]
+    async fn file_structure_support_methods() {
+        let data: String = rand::thread_rng()
+            .sample_iter(&Alphanumeric)
+            .take(512)
+            .map(char::from)
+            .collect();
+        let tmp = assert_fs::TempDir::new().unwrap();
+        let tmp_file = tmp.child("foo");
+        tmp_file.touch().unwrap();
+        tmp_file.write_binary(data.as_bytes()).unwrap();
+
+        let file = File::from_path(&tmp_file).await.unwrap();
+        assert_eq!(file.name(), "foo");
+        let file = file.with_name("bar");
+        assert_eq!(file.name(), "bar");
+        assert_eq!(file.size().await, 512);
+    }
+
+    #[tokio::test]
+    async fn file_structure_into_encrypted() {
+        let secret = "secret";
+        let data: String = rand::thread_rng()
+            .sample_iter(&Alphanumeric)
+            .take(512)
+            .map(char::from)
+            .collect();
+        let tmp = assert_fs::TempDir::new().unwrap();
+        let tmp_file = tmp.child("foo");
+        tmp_file.touch().unwrap();
+        tmp_file.write_binary(data.as_bytes()).unwrap();
+
+        let file = Box::new(File::from_path(&tmp_file).await.unwrap());
+        let encrypted = File::from_path(&tmp_file)
+            .await
+            .unwrap()
+            .into_encrypted(secret)
+            .await
+            .unwrap();
+        // original file is intact
+        tmp_file.assert(predicates::str::contains(&data));
+        assert_ne!(file.data(), encrypted.data());
+        // file prev encrypted is the same decrypted one
+        assert_eq!(file.data(), encrypted.prev.unwrap().data());
+    }
+
+    #[tokio::test]
+    async fn file_encrypted_structure_from_bytes_encoded() {
+        let secret = "secret";
+        let data = "I'm a decrypted string".to_owned();
+        let tmp = assert_fs::TempDir::new().unwrap();
+        let tmp_file = tmp.child("foo");
+        tmp_file.touch().unwrap();
+        tmp_file.write_binary(data.as_bytes()).unwrap();
+
+        let encoded_data = File::from_path(&tmp_file)
+            .await
+            .unwrap()
+            .into_encrypted(secret)
+            .await
+            .unwrap()
+            .to_encoded_data();
+        let encrypted =
+            EncryptedFile::from_bytes_encoded(encoded_data.inner.as_bytes(), "nameless")
+                .await
+                .unwrap();
+        assert_ne!(encrypted.data(), data.as_bytes());
+        assert!(matches!(encrypted.prev, None));
+    }
+
+    #[tokio::test]
+    async fn file_encrypted_structure_into_decrypted() {
+        let secret = "secret";
+        let data = "I'm a decrypted string".to_owned();
+        let tmp = assert_fs::TempDir::new().unwrap();
+        let tmp_file = tmp.child("foo");
+
+        tmp_file.touch().unwrap();
+        tmp_file.write_binary(data.as_bytes()).unwrap();
+
+        let encrypted_file = File::from_path(&tmp_file)
+            .await
+            .unwrap()
+            .into_encrypted(secret)
+            .await
+            .unwrap();
+        let encoded_encrypted_data = encrypted_file.to_encoded_data();
+        let decrypted_file = encrypted_file.into_decrypted(secret).await.unwrap();
+
+        assert_eq!(decrypted_file.data(), data.as_bytes());
+    }
+
+    #[tokio::test]
+    async fn file_encryption_header_data() {
+        let secret = "secret";
+        let data = "Matthew McConaughey".to_owned();
+        let tmp = assert_fs::TempDir::new().unwrap();
+        let tmp_file = tmp.child("foo");
+
+        tmp_file.touch().unwrap();
+        tmp_file.write_binary(data.as_bytes()).unwrap();
+
+        let encoded_encrypted_data = File::from_path(&tmp_file)
+            .await
+            .unwrap()
+            .into_encrypted(secret)
+            .await
+            .unwrap()
+            .to_encoded_data();
+        let (nonce, rest) = parse_encryption_header(
+            base64::decode(encoded_encrypted_data.inner)
+                .unwrap()
+                .as_slice(),
+        )
+        .unwrap();
+        let nonce: [u8; 12] = nonce.try_into().unwrap();
+        let decrypted_data = decrypt_aes256_u12nonce(secret.as_bytes(), &rest, &nonce).unwrap();
+        assert_eq!(decrypted_data, data.as_bytes());
+    }
+
+    #[tokio::test]
+    async fn file_temp_fs_file_deleted_on_drop() {
+        let data = "Matthew McConaughey".to_owned();
+        let file = File::from_bytes(data.as_bytes()).await.unwrap();
+        let path = file.path.clone();
+        assert!(tokio::fs::metadata(&path).await.unwrap().is_file());
+        {
+            file;
+        }
+        let not_found = tokio::fs::metadata(&path).await.unwrap_err().kind();
+        assert!(matches!(not_found, std::io::ErrorKind::NotFound));
+    }
+}
